package org.dromara.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.dromara.system.domain.StudentInfo;
import org.dromara.system.domain.SysUser;
import org.dromara.system.domain.bo.StudentInfoBo;
import org.dromara.system.domain.vo.StudentInfoVo;
import org.dromara.common.mybatis.core.mapper.BaseMapperPlus;

import java.util.List;

/**
 * 学生档案信息Mapper接口
 *
 * @author Lion Li
 * @date 2023-08-16
 */
public interface StudentInfoMapper extends BaseMapperPlus<StudentInfo, StudentInfoVo> {


    @InterceptorIgnore(tenantLine = "1")
    StudentInfoVo selectStudentInfoByStudentId(Long studentId);

    @InterceptorIgnore(tenantLine = "1")
    Page<StudentInfoVo> selectStudentInfoPage(@Param("page")Page<Object> build, @Param(Constants.WRAPPER) Wrapper<StudentInfo> queryWrapper);

    @InterceptorIgnore(tenantLine = "1")
    List<StudentInfoVo> selectStudentInfo(@Param(Constants.WRAPPER)QueryWrapper<StudentInfo> lqw);
    @InterceptorIgnore(tenantLine = "1")
    StudentInfoVo selectStudentInfoById(Long id);
}

